/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import java.util.Optional;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(CollaborationController.PATH)
@RequiredArgsConstructor
public class CollaborationController {

	private final CollaborationModelAssembler assembler;
	private final CollaborationService service;

	static final String PATH = "/api/collaborations"; // NOSONAR

	@GetMapping("/{collaborationId}")
	public ResponseEntity<EntityModel<Collaboration>> getById(@PathVariable String collaborationId) {
		return ResponseEntity.of(Optional.of(service.getById(collaborationId)).map(assembler::toModel));
	}

	@RestController
	@RequestMapping(CollaborationByVorgangController.PATH)
	@RequiredArgsConstructor
	public static class CollaborationByVorgangController {

		static final String PATH = "/api/vorgangs";

		private final CollaborationModelAssembler assembler;
		private final CollaborationService service;

		@GetMapping("/{vorgangId}/collaborations")
		public CollectionModel<EntityModel<Collaboration>> getAllByVorgangId(@PathVariable String vorgangId) {
			return assembler.toCollectionModel(service.getCollaborations(vorgangId), vorgangId);
		}
	}
}
