/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import java.util.Collection;
import java.util.List;
import java.util.UUID;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.alfa.common.user.UserId;
import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;

@Builder
@Getter
@EqualsAndHashCode
class TestEntity {

	@LinkedResource(controllerClass = TestIdController.class)
	private String linkedResource;
	@LinkedResource(controllerClass = TestIdController.class)
	private String testId;

	private String foo;
	private Collection<String> collectionField;
	private String[] arrayField;

	@LinkedUserProfileResource
	private UserId user;

	@LinkedUserProfileResource
	private String differentUserId;
}

@RequestMapping(TestEntityController.PATH)
@RestController
class TestEntityController {

	static final String PATH = "/api/entity";
	static final String GET_ALL = "/all";

	@GetMapping
	public EntityModel<TestEntity> getTestEntity() {
		return EntityModel.of(TestEntityTestFactory.create());
	}

	@GetMapping(GET_ALL)
	public CollectionModel<EntityModel<TestEntity>> getAll() {
		return CollectionModel.of(List.of(EntityModel.of(TestEntityTestFactory.create())));
	}
}

@RequestMapping(TestIdController.PATH)
class TestIdController {

	static final String PATH = "/api/test";

	static final String USER_REL = "user";
	static final String DIFFERENT_USER_REL = "differentUser";
	static final String ID_REL = "test";
	static final String LINKED_RESOURCE_REL = "linkedResource";

}

class TestEntityTestFactory {

	static final String USER = UUID.randomUUID().toString();
	static final String DIFFERENT_USER_ID = UUID.randomUUID().toString();
	static final String ID = UUID.randomUUID().toString();
	static final String LINKED_RESOURCE = UUID.randomUUID().toString();

	public static TestEntity create() {
		return createBuilder().build();
	}

	public static TestEntity.TestEntityBuilder createBuilder() {
		return TestEntity.builder()
				.testId(ID)
				.linkedResource(LINKED_RESOURCE)
				.user(UserId.from(USER))
				.differentUserId(DIFFERENT_USER_ID)
				.foo("bar");
	}
}
