/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.export;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.io.OutputStream;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class ExportVorgangControllerTest {

	@Spy
	@InjectMocks
	private ExportVorgangController controller;
	@Mock
	private ExportRemoteService exportRemoteService;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestExportVorgang {

		public static final String VORGANG_ID = VorgangHeaderTestFactory.ID;
		public static final String EXPORTED_VORGANG_FILENAME = UUID.randomUUID().toString();

		@Mock
		private ExportedVorgangFile exportedVorgangFile;

		@BeforeEach
		void init() {
			when(exportedVorgangFile.getFileName()).thenReturn(EXPORTED_VORGANG_FILENAME);
			when(exportRemoteService.exportVorgang(VORGANG_ID)).thenReturn(exportedVorgangFile);
		}

		@Test
		void shouldExportVorgang() throws Exception {
			doRequest();

			verify(exportRemoteService).exportVorgang(VORGANG_ID);
		}

		@Test
		void shouldHaveContentDispositonHeader() throws Exception {
			doRequest().andExpect(header().string(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + EXPORTED_VORGANG_FILENAME));
		}

		@Test
		void shouldHaveContentTypeHeader() throws Exception {
			doRequest().andExpect(header().string(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_OCTET_STREAM.toString()));
		}

		@Test
		void shouldWriteFileToOutputStream() throws Exception {
			doRequest();

			verify(exportedVorgangFile).writeToOutputStream(any(OutputStream.class));
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(asyncDispatch(
							mockMvc.perform(get(ExportVorgangController.PATH + "/" + VorgangHeaderTestFactory.ID)
									.header(HttpHeaders.ACCEPT, MediaType.APPLICATION_OCTET_STREAM_VALUE)).andReturn()))
					.andExpect(status().isOk());
		}
	}

}